#!/usr/bin/env python

import struct
from HCI_Codes import *
from HCIEvents import HCI_Vendor_Events, HCI_Events
from time import sleep
import time

def senao_hci_read(bt):
    if bt.isOpen():
        packet_type = bt.read()
        try:
            print ""
            print "Found", pkt_types[packet_type], "Packet"
            if packet_type == PKTTYPES['Event']:
                event_code=bt.read()
                try:
                    print evt_codes[event_code], "Event Code"
                    if event_code == EVTCODES['Low Energy']:
                        print "Should not get here. HCI is not supposed to return such events."
                        sub_evt_code = bt.read()
                        try:
                            print sub_evt_codes[sub_evt_code], "Subevent Code"
                        except KeyError:
                            print "Unknown Subevent Code"

                    elif event_code == EVTCODES['Vendor Specific']:
                        #event = bt.read(size=3)  # length + event OpCode
                        evt_len, evt_opcode = struct.unpack('<BH',bt.read(size=3))
                        print "Data length:" + str(evt_len)
                        print "Data Code:" + hex(evt_opcode)
                        HCI_Vendor_Events().lookup(evt_opcode)(evt_len-2, bt)
                    elif event_code == EVTCODES['Command Complete BT']:
                        #evt_len, packets, evt_opcode, evt_status, bd_addr=struct.unpack('<BBHB6B',bt.read(size=11))
                        evt_len, packets, evt_opcode = struct.unpack('<BBH',bt.read(size=4))
                        #evt_len, packets, evt_opcode, evt_status, bdaddr = struct.unpack('<BBHB6s',bt.read(size=11))
                        print "Data length:" + str(evt_len)
                        print "Data packets:" + str(packets)
                        print "Data Code:" + hex(evt_opcode)
                        if hex(evt_opcode) == "0x201f":
                            return HCI_Events().lookup(evt_opcode)(evt_len-3, bt)
                        else:
                            HCI_Events().lookup(evt_opcode)(evt_len-3, bt)
                        #print "evt_opcode:" + str(evt_opcode)
                        #print "status:" + str(evt_status)
                        #print bdaddr.encode("hex")
                        #print "opcode:" + bt.read()
                        #print "status:" + bt.read()
                        #print "bdaddr:" + bt.read()
                    else:
                        print "Should not get here. HCI is not supposed to return such events."
                        print "Don't know how to handle this Event Code"
                        print "Event Code:",hex(struct.unpack('<B',event_code)[0])
                        #evt_len = struct.unpack('<B',bt.read())[0]
                        #print "Flushing ",str(evt_len)," bytes"
                        #bt.read(size=evt_len)

                except:
                    print "Unknown Event Code: ", struct.unpack('<B',event_code)
        except:
            sys.exit("Senao HCI Terminated.")

def print_packet(packet,message):
    count=0
    count2=0
    log=[]
    last=len(packet)
    print ""
    print message
    for c in packet:
        log.append("{:02x}".format(ord(c)))
        count=count+1
        count2=count2+1
        if(count == 8 or count2 == last):
            print ":".join(log)
            count=0
            log=[]
    print ""

def RdLocalSupFeatures(bt, *byte):
    command = PKTTYPES['Command'] + '\x03' + '\x10' + '\x00'
    bt.write(command)
    print "Read Local Support Features"

def RdBdAddr(bt, *byte):
    command = PKTTYPES['Command'] + '\x09' + '\x10' + '\x00'
    bt.write(command)
    print "Read BD Address."

def RdRSSI(bt, *byte):
    command = PKTTYPES['Command'] + '\x05' + '\x14' + '\x00'
    bt.write(command)
    print "Read RSSI."

def LERdLocalSupFeatures(bt, *byte):
    command = PKTTYPES['Command'] + '\x03' + '\x20' + '\x00'
    bt.write(command)
    print "Read LE Local Support Features"

def LERdWhiteListSize(bt, *byte):
    command = PKTTYPES['Command'] + '\x0F' + '\x20' + '\x00'
    bt.write(command)
    print "Read LE White List Size"

def LEClrWhiteList(bt, *byte):
    command = PKTTYPES['Command'] + '\x10' + '\x20' + '\x00'
    bt.write(command)
    print "Clear LE White List"

def LEReceiverTest(bt, byte):
    command = PKTTYPES['Command'] + '\x1d' + '\x20' + '\x01'
    for e in byte: 
        command = command + str(unichr(e)) 
    print_packet(command,"LE Receiver Test")
    bt.write(command)
    print "LE Receiver Test"

def LETransmitterTest(bt, byte):
    command = PKTTYPES['Command'] + '\x1e' + '\x20' + '\x03'
    for e in byte:                                                                             
        command = command + str(unichr(e))                                   
    print_packet(command,"LE Transmitter Test")
    bt.write(command)
    print "LE Transmitter Test"

def LETestEnd(bt, *byte):
    command = PKTTYPES['Command'] + '\x1f' + '\x20' + '\x00'
    bt.write(command)
    print "LE Test End"

def GAP_DeviceInit(bt, *byte):
    '''
        GAP Device Initialization

        Packet is as follow:
            - packet type (1 byte)
            - GAP command (2 bytes)
            - length of data (1 byte)
            - Data:
                i)   GAP Profile Role (1 byte) (Central)
                ii)  Maximum Scan Responses to accept (1 byte) (0x00-0xFF)
                iii) IRK (16 bytes) if all zeros, randomly generated
                iv)  CSRK (16 bytes) if all zeros, randomly generated
                v)   Signature counter initial value (4 bytes)
    '''
    command = PKTTYPES['Command'] + get_gap_cmd('GAP Device Initialization') + struct.pack('B',struct.calcsize('BB16s16sL'))
    command = command + struct.pack('BB16s16sL',struct.unpack('<B',GAPROLES['Central'])[0],3,'\x00','\x00',1)
    #print command
    bt.write(command)
    print "GAP Device Init."

def RAW_Write(bt, byte):
    strbuf=''
    for e in byte:
        print unichr(e)
        strbuf = strbuf + unichr(e)
    print strbuf
    bt.write(strbuf)
    print "RAW Write"


def Senao_GAP_DeviceInit(bt):
    #initialize as peripheral
    #Send_GAP_DeviceInit 0x04, 3, 0, 0, 1
    command = PKTTYPES['Command'] + get_gap_cmd('GAP Device Initialization') + '\x26'
    command = command + struct.pack('BB',struct.unpack('<B',GAPROLES['Peripheral'])[0],3)
    command = command + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00'
    command = command + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00' + '\x00'
    command = command + '\x01' + '\x00' + '\x00' + '\x00'
    print_packet(command,"GAP Device Init")
    bt.write(command)


def Senao_GATT_AddService(bt):
    #Add primary service
    command = PKTTYPES['Command'] + get_gatt_cmd('GATT Add Service') + '\x04'
    command = command + '\x00' + '\x28' + '\x04' + '\x00'
    print_packet(command,"GATT Add Service")
    bt.write(command)

def Senao_GATT_AddAttribute(bt, byte):
    #Add primary service
    command = PKTTYPES['Command'] + get_gatt_cmd('GATT Add Attribute') + '\x03'
    command = command + str(byte[0]) + str(byte[1]) + str(byte[2])
    print_packet(command,"GATT Add Attribute")
    bt.write(command)

def Senao_ATT_FindInfoReq(bt, byte):
    #Senao_Att_FindInfoReq
    command = PKTTYPES['Command'] + get_att_cmd('ATT Find Information Request') + '\x06'
    for i in range(0,6,1):
        command = command + str(byte[i])
    print_packet(command,"Att Find Info Req")
    bt.write(command)

def Senao_GAP_UpdateAdvertisingData(bt, byte):
    #Update advertising data
    command = PKTTYPES['Command'] + get_gap_cmd('GAP Update Advertising Data') + '\x0a'
    for i in range(0,10,1):
        command = command + str(byte[i])
    print_packet(command,"GAP Update Advertising Data")
    bt.write(command)

def Senao_GAP_MakeDiscoverable(bt, byte):
    #Update advertising data
    command = PKTTYPES['Command'] + get_gap_cmd('GAP Make Discoverable') + '\x0a'
    for i in range(0,10,1):
        command = command + str(byte[i])
    print_packet(command,"GAP Make Discoverable")
    bt.write(command)

def Senao_HCI_Reset(bt):
    command = '\x01' + '\x03' + '\x0c' + '\x00'
    print_packet(command,"Senao HCI Reset")
    bt.write(command)

def Senao_HCI_EXT_ModemTestRx(bt):
    command = '\x01' + '\x0a' + '\xfc' + '\x01' + '\x00'
    print_packet(command,"Senao HCI EXT ModemTestRx")
    bt.write(command)

def Senao_HCI_Read_RSSI(bt):
    command = '\x01' + '\x05' + '\x14' + '\x02' + '\x00' + '\x00'
    print_packet(command,"Senao HCI Read RSSI")
    bt.write(command)

def NP_GATT_DB_OFF_CHIP(bt, byte):
    print "NP_GATT_DB_OFF_CHIP"
    #Initialize as periphearal
    Senao_GAP_DeviceInit(bt)
    #Add primary service
    Senao_GATT_AddService(bt)
    #Add characteristic declaration
    Senao_GATT_AddAttribute(bt,['\x03', '\x28', '\x03'])
    #Device name
    Senao_GATT_AddAttribute(bt,['\x00', '\x2a', '\x03'])
    sleep(0.1)
    #Add primary service 2
    Senao_GATT_AddService(bt)
    #Add characteristic declaration - SimpleBleChar1
    Senao_GATT_AddAttribute(bt,['\x03', '\x28', '\x03'])
    #Char 1 value
    Senao_GATT_AddAttribute(bt,['\xf1', '\xff', '\x03'])
    #Char 1 description
    Senao_GATT_AddAttribute(bt,['\x01', '\x29', '\x03'])
    sleep(0.1)
    #Send_ ATT_FindInfoReq
    Senao_ATT_FindInfoReq(bt,['\xfe', '\xff', '\x01', '\x00', '\xff', '\xff'])
    #Update advertising data
    Senao_GAP_UpdateAdvertisingData(bt,['\x01', '\x08', '\x02', '\x01', '\x04', '\x05', '\x05', '\x05', '\x07', '\x07'])
    #Update scan respon
    Senao_GAP_UpdateAdvertisingData(bt,['\x00', '\x08', '\x02', '\x01', '\x04', '\x05', '\x02', '\x02', '\x02', '\x02'])
    #Make discoverable: GAP_ADTYPE_ADV_IND
    Senao_GAP_MakeDiscoverable(bt,['\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x00', '\x07', '\x00'])

def SCRIPT_RX_TEST(bt, byte):
    print "SCRIPT_RX_TEST"
    Senao_HCI_Reset(bt)
    #Wait_HCI_Command_Reset_Event Complete
    sleep(5)
    senao_hci_read(bt)

    Senao_HCI_EXT_ModemTestRx(bt)
    senao_hci_read(bt)

    Senao_HCI_Read_RSSI(bt)
    #Wait HCI_Command_Read_RSSI_Event Complete
    sleep(5)
    senao_hci_read(bt)
    bt.close()

def SCRIPT_PER_TEST(bt, byte):                                             
    print "SCRIPT_PER_TEST"
    start_time=time.time()
    LEReceiverTest(bt, byte)
    senao_hci_read(bt)
    Senao_HCI_Read_RSSI(bt)
    senao_hci_read(bt)
    sleep(2)
    end_time=time.time()
    total_packet = round((end_time-start_time)/0.000625)
    LETestEnd(bt, byte)
    receive_packet=senao_hci_read(bt)
    print receive_packet
    print "Total: ", total_packet
    print "received:", receive_packet
    print "PER: ", format((total_packet-receive_packet)/total_packet, '.2%')
    bt.close()
    
def HCI_EXT_SetTxPowerCmd(bt, byte):
    command = PKTTYPES['Command'] + '\x01' + '\xFC' + '\x01'
    for e in byte:
        command = command + str(unichr(e))
    print_packet(command,"Ext Set Tx Power")
    bt.write(command)
    print "Ext Set Tx power"

def HCI_EXT_EndModemTestCmd(bt, byte):
    command = '\x01' + '\x0B' + '\xFC' + '\x00'
    print_packet(command,"Senao EndModemTestCmd")
    bt.write(command)

def EnterTxModemTest(bt, byte):
    GAP_DeviceInit(bt)
    command = '\x01' + '\x08' + '\xFC' + '\x02'
    for e in byte:                                                                                                   
        command = command + str(unichr(e))
    print_packet(command,"Senao EnterTxModemTest")
    print "<<press Q to end loop>>"
    bt.write(command)
    while(1):
        if(raw_input() == "q"):
            HCI_EXT_EndModemTestCmd(bt, byte)
            break;
    bt.close()

OPCODES={'0x1003':RdLocalSupFeatures,
         '0x1009':RdBdAddr,
         '0x1405':RdRSSI,
         '0x2003':LERdLocalSupFeatures,
         '0x200f':LERdWhiteListSize,
         '0x2010':LEClrWhiteList,
         #'0x2011':LEAddDeviceToWhiteList,
         #'0x2012':LERemoveDeviceFromWhiteList,
         #'0x2013':LEConnectionUpdate,
         '0x201d':LEReceiverTest,
         '0x201e':LETransmitterTest,
         '0x201f':LETestEnd,
         '0xfc01':HCI_EXT_SetTxPowerCmd,
         '0xfe00':GAP_DeviceInit,
         '0xffff':RAW_Write,
         '0xffff0':NP_GATT_DB_OFF_CHIP,
         '0xffff1':SCRIPT_RX_TEST,
         '0xffff2':EnterTxModemTest,
         '0xffff3':SCRIPT_PER_TEST}

